import { getHashString, adaptiveFilesize } from "./utils";

export async function listDispatcher(request, env, ctx) {
    const url = new URL(request.url);
    const headers = request.headers;
    
    const bucket = env.EXP_DATA;
    const signatureBase = env.SIGNATURE_BASE;
    const adminSecret = env.ADMIN_SECRET;
    const badRequest = new Response(null, { status: 400 });
    
    const token = url.searchParams.get('accessToken') || headers.get('x-access-token');
    if (token !== adminSecret)
        return new Response(null, { status: 401, statusText: 'Unauthorized' });
    
    const resType = url.searchParams.get('resType') || headers.get('x-res-type') || 'html';
    if (!['json', 'html'].includes(resType))
        return badRequest;

    try {
        const type = url.searchParams.get('type') || headers.get('x-type') || 'prefix';
        let pattern = url.searchParams.get('pattern') || headers.get('x-pattern') || '';
        
        let objects = [];
        let truncated = true;
        let cursor;
        switch (type) {
            case 'prefix':
                while (truncated) {
                    const res = await bucket.list({ prefix: pattern, cursor });
                    objects = objects.concat(res.objects);
                    truncated = res.truncated;
                    cursor = res.cursor;
                }
                break;
            case 'suffix':
                pattern += '$';
            case 'regex':
                while (truncated) {
                    const res = await bucket.list({ cursor: cursor });
                    objects = objects.concat(res.objects);
                    truncated = res.truncated;
                    cursor = res.cursor;
                }
                pattern = new RegExp(pattern);
                objects = objects.filter(obj => obj.key.match(pattern));
                break;
            default:
                return badRequest;
        }

        const timestamp = Date.now();
        const baseUrl = `${url.protocol}//${url.hostname}:${url.port}`;
        const res = await Promise.all(objects.map(async object => {
            const { key, etag, size, uploaded } = object;
            const hash = await getHashString(`${key}${timestamp}${signatureBase}`, 'SHA-1');
            return { key, etag, size, uploaded,
                     presignedUrl: `${baseUrl}/download/${key}?signature=${hash}&timestamp=${timestamp}` };
        }));
        console.log(objects[0]);
        if (resType === 'json')
            return new Response(JSON.stringify(res), { status: 200 });
        else if (resType === 'html') {
            // Generate a table of links with URLs, sizes, and upload dates
            const body = res.map(obj => `
                <tr>
                    <td><a href="${obj.presignedUrl}">${obj.key}</a></td>
                    <td>${adaptiveFilesize(obj.size)}</td>
                    <td>${obj.uploaded.toISOString()}</td>
                </tr>
            `).join('');
            return new Response(`
                <html>
                    <head>
                        <title>Object List</title>
                        <meta charset="utf-8">
                    </head>
                    <body>
                        <table>
                            <tr>
                                <th>Key</th>
                                <th>Size</th>
                                <th>Uploaded</th>
                            </tr>
                            ${body}
                        </table>
                    </body>
                </html>
            `, { status: 200, headers: { 'content-type': 'text/html' } });
        }

    } catch (e) { console.log(e); return badRequest; }
}

export default listDispatcher;